unit launchthreads;

interface

uses
  Classes, Windows, SysUtils, forms, stdctrls,ScktComp2, audioio,
  cyclicBuffer,generic;

type

  TLaunchReceiverThread = class(TThread)
  private
   AudioOut: TAudioOut;
  protected
    procedure Execute; override;
  public
    constructor create(_AudioOut: TAudioOut);
    destructor Destroy; override;
  end;


  TLaunchSenderThread = class(TThread)
  private
   AudioIn: TAudioIn;
  protected
    procedure Execute; override;
  public
    constructor create(_AudioIn: TAudioIn);
    destructor Destroy; override;
  end;


  TSocketHandlerThread = class(TThread)
  private
   buffer: TCyclicBuffer;
   frmOwner: TForm;
   outputBuffer: array[0..buffersize-1] of byte;
   procedure callMainHandleReceivedBuffer;
  protected
    procedure Execute; override;
  public
    constructor create(_frmOwner: TForm);
    destructor Destroy; override;
    procedure handleReceivedBuffer(receivedBuffer: pbyte; size: integer);
  end;


implementation

uses
   main;

{constructor}
constructor TLaunchReceiverThread.create(_AudioOut: TAudioOut);
begin
   inherited create(true); // create but don't start running yet
   AudioOut := _AudioOut;
end;

{destructor}
destructor TLaunchReceiverThread.Destroy;
begin
   inherited destroy;
end;


{main thread entry point}
procedure TLaunchReceiverThread.Execute;
begin
   AudioOut.paused := true;
   AudioOut.start;
   AudioOut.paused := false;
end;





{constructor}
constructor TLaunchSenderThread.create(_AudioIn: TAudioIn);
begin
   inherited create(true); // create but don't start running yet
   AudioIn := _AudioIn;
end;

{destructor}
destructor TLaunchSenderThread.Destroy;
begin
   inherited destroy;
end;


{main thread entry point}
procedure TLaunchSenderThread.Execute;
begin
   AudioIn.paused := true;
   AudioIn.start;
   AudioIn.paused := false;
end;




{constructor}
constructor TSocketHandlerThread.create(_frmOwner: TForm);
begin
   inherited create(true); // create but don't start running yet
   frmOwner := _frmOwner;
   buffer:= TCyclicBuffer.create(1024 * 1024 * 5);
end;

{destructor}
destructor TSocketHandlerThread.Destroy;
begin
   buffer.free;
   inherited destroy;
end;


{main thread entry point}
procedure TSocketHandlerThread.Execute;
begin
   while running do
   begin
      buffer.popBuffer(@outputBuffer[0], buffersize);
      synchronize(callMainHandleReceivedBuffer);
   end;
end;

procedure TSocketHandlerThread.callMainHandleReceivedBuffer;
begin
   frmMain.handleReceivedBuffer(pchar(@outputBuffer), buffersize);
end;


procedure TSocketHandlerThread.handleReceivedBuffer(receivedBuffer: pbyte; size: integer);
begin
   buffer.pushBuffer(receivedBuffer, size);
end;



end.
